use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007003
use Test::Spelling 0.12;
use Pod::Wordlist;

set_spell_cmd('aspell list');
add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
afPuUsSedvhx
Accomazzi
Alexandre
Andreas
Asplund
Betts
Bochner
BooK
Buenzli
Chamas
Coppit
Dalgleish
Dubois
Dunkin
Duret
Dvornik
Eldridge
Gertjan
Graaff
Greab
Guenther
Gurusamy
Gustafsson
Hakanson
Harald
Hedlund
Hoblitt
Hwa
INOUE
Joao
Joerg
Kaminsky
Kartik
Katsuhiro
Kebsch
Keiichiro
Kilzer
Klar
Koster
König
KONISHI
Kronengold
Krüger
Kubb
Laker
Langfeldt
Langheinrich
Liam
Lindley
Lotterer
Lutz
MacEachern
Macdonald
Marko
Markus
Martijn
McCauley
Melchner
Moshe
Murrell
Nagano
Nicolai
Nierstrasz
Olly
Oosten
Panchenko
Pimlott
Pon
Quaranta
Radoslaw
Radu
Rai
Rezic
Sarathy
Schilli
Schinder
Shirazi
Skyttä
Slaven
Spafford
Stosberg
Subbarao
Takanori
Thoennes
Thurn
Tilly
Ugai
Unger
VanHeyningen
Vandewege
Ville
Yitzchak
Yoshinari
Zajac
Zakharevich
Zielinski
Zoest
CPAN
CGI
UserAgent
UA
RobotUA
NTLM
NNTP
TCP
WireShark
Yee
shildreth
Newby
Mailto
peterm
de
erik
